using System;
using ClearImage;
using System.IO;
using ClearMicr;

namespace nsCiFnc
{
	/// <summary>
	/// Summary description for ClearMicrFnc.
	/// </summary>
	public class ClearMicrFnc
	{
		static System.Threading.Mutex mut =  new System.Threading.Mutex();	// Prevents reentrancy

        ClearMicr.CcMicrReader MicrReader = null; 
		public ClearMicrFnc()
		{
 		MicrReader = new ClearMicr.CcMicrReader(); 
		}
		//###################################################################
		public string ReadMICR (string sFile, EMicrReaderFlags flags, int maxMicrs)
		{
			try 
			{ 
				mut.WaitOne();
				EMicrReaderFlags nFlags  = 0;
					// Open Image
	  		MicrReader.Image.Open(sFile, 1); 
					// Do actual reading 
        MicrReader.MaxMicrCount = (short) maxMicrs;
 	  		MicrReader.Flags = flags;
				MicrReader.FindMICR();
				string s = ""; 
				for (int i = 1; i <= MicrReader.MicrCount; i++) 
					{ 
					ClearMicr.CcMicr Micr = MicrReader.get_MicrLine(i);
					s  += AddMICR (Micr, i); } 
				if ((s == "")) 		{ s = "NO MICR"; 	} 
				return  s; 
			} 
			catch (Exception ex) 
			{ 
				return ("ERROR: " + ex.ToString()); 
			} 
			finally
			{
				mut.ReleaseMutex();
			}
		}

        String[] rotText = {"", "None", "Upside Down", "Left", "Right", "Unknown"}; 
		//###################################################################
		public string  ExtractCheck (string sFile, string sOut, ClearMicr.EMicrReaderFlags flags)
		{
			try 
			{ 
				mut.WaitOne();
					// Open Image
	  		MicrReader.Image.Open(sFile, 1); 
	  		MicrReader.Flags = flags;
					// Do actual reading 
				CiImage oImage;
				oImage = MicrReader.ExtractCheck();
				if (oImage  == null) return "";
				oImage.SaveAs (sOut, ClearImage.EFileFormat.ciTIFF);
				String s = ""; 
				for (int i = 1; i <= MicrReader.MicrCount; i++) 
					{ 
					 ClearMicr.CcMicrInfo Info = MicrReader.get_MicrLine(i).Document;
                     s = String.Format("Extracted Document \r\n at {0}:{1}-{2}:{3}  Skew: {4:F}deg   Rotation: {5}",
                     Info.Left, Info.Top, Info.Right, Info.Bottom, Info.Skew,  rotText[(int)Info.Conf]);
                     s += Environment.NewLine + Environment.NewLine;
                 	s  += AddMICR (MicrReader.get_MicrLine(i), i); } 
				if ((s == "")) 		{ s = "<NO MICR>"; 	} 
				return  s; 
			} 
			catch (Exception ex) 
			{ 
				return "<" + ex.Message + ">"; 
			} 
			finally
			{
				mut.ReleaseMutex();
			}
		}

		//###################################################################
		//    MICR display examples
		//###################################################################
		private string AddMICRFields (ClearMicr.CcMicrInfo Info) 
			{
			string s = "";
			if (!Info.IsRead) return s;
			string sName = Info.Name;
			string sText = Info.TextANSI;
			double dConf = Info.Conf;
			bool   bAba  = Info.IsAbaCompliant;
			s = s + "  " + 
			String.Format("{0,-15} = '{1,-15}'  CONF={2:#0}%  {3}\r\n",
					sName, sText, dConf, (bAba ? "ABA Compliant" : ""));
			return s;
			}

		private string AddMICR (ClearMicr.CcMicr Micr, long i) 
		{ 
		  string s = "";
			s = s + "MICR #" + i + " Text:" + Micr.Info.TextRaw + "\r\n";
			s = s + "FIELDS" + "\r\n";
			s += AddMICRFields( Micr.AuxOnUs);
			s += AddMICRFields( Micr.EPC);
			s += AddMICRFields( Micr.Routing);
			s += AddMICRFields( Micr.RoutingChecksum);
			s += AddMICRFields( Micr.OnUs);
			s += AddMICRFields( Micr.Amount);
			s += AddMICRFields( Micr.Account);
			s += AddMICRFields( Micr.CheckNumber);
			s += AddMICRFields( Micr.AuxOnUs);
			return s;
		} 
	}
}
